"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Lodash = require("lodash");
const router_1 = require("./router");
const resolver_1 = require("./resolver");
const router_2 = require("./router");
const Fs = require("fs");
const logger_1 = require("./logger");
const validator_1 = require("./validator");
const Http = require("http");
/**
 * Create instance of KambojaJS application
 */
class Kamboja {
    /**
     * Create instance of KambojaJS application
     * @param engine KambojaJS engine implementation
     * @param opt KambojaJS option
     */
    constructor(engine, opt) {
        this.engine = engine;
        let override;
        if (typeof opt === "string")
            override = { rootPath: opt };
        else
            override = opt;
        let idResolver = new resolver_1.DefaultIdentifierResolver();
        let pathResolver = new resolver_1.DefaultPathResolver(override.rootPath);
        let resolver = new resolver_1.DefaultDependencyResolver(idResolver, pathResolver);
        let storage = new router_1.MetaDataLoader(idResolver, pathResolver);
        let options = Lodash.assign({
            skipAnalysis: false,
            controllerPaths: ["controller"],
            modelPath: Kamboja.defaultModelPath,
            autoValidation: true,
            rootPath: process.cwd(),
            showLog: "Info",
            validators: [],
            middlewares: [],
            facilities: [],
            identifierResolver: idResolver,
            pathResolver: pathResolver,
            dependencyResolver: resolver,
            metaDataStorage: storage
        }, override);
        this.options = options;
        Kamboja.facade = options;
        this.storage = this.options.metaDataStorage;
    }
    static getFacade() {
        return Kamboja.facade;
    }
    set(key, value) {
        this.options[key] = value;
        return this;
    }
    get(key) {
        return this.options[key];
    }
    /**
     * Add middleware
     * @param factory factory method that will be call after KambojaJS application initialized
     * @returns KambojaJS application
     */
    use(middleware) {
        this.options.middlewares.push(middleware);
        return this;
    }
    apply(facility) {
        if (typeof facility == "string") {
            try {
                let fac = this.options.dependencyResolver.resolve(facility);
                this.options.facilities.push(fac);
                fac.apply(this);
            }
            catch (e) {
                throw new Error(`Unable to instantiate ${facility} as Facility`);
            }
        }
        else {
            this.options.facilities.push(facility);
            facility.apply(this);
        }
        return this;
    }
    isFolderProvided() {
        let result = true;
        //controller
        this.options.controllerPaths.forEach(x => {
            let path = this.options.pathResolver.resolve(x);
            if (!Fs.existsSync(path)) {
                result = false;
                this.log.error(`Controller path [${x}] provided in configuration is not exist`);
            }
        });
        //model
        let modelPath = this.options.pathResolver.resolve(this.options.modelPath);
        if (!Fs.existsSync(modelPath) && this.options.modelPath != Kamboja.defaultModelPath) {
            this.log.error(`Model path [${this.options.modelPath}] provided in configuration is not exist`);
            result = false;
        }
        return result;
    }
    generateRoutes(controllerMeta) {
        let route = new router_2.RouteGenerator(this.options.identifierResolver, controllerMeta);
        let infos = route.getRoutes();
        if (infos.length == 0) {
            let paths = this.options.controllerPaths.join(",");
            this.log.error(`No controller found in [${paths}]`);
        }
        return infos;
    }
    analyzeRoutes(infos) {
        let routeAnalyzer = new router_2.RouteAnalyzer(infos);
        let analysis = routeAnalyzer.analyse();
        for (let item of analysis) {
            if (item.type == "Warning")
                this.log.warning(item.message);
            else
                this.log.error(item.message);
        }
        if (analysis.some(x => x.type == "Error")) {
            return false;
        }
        let validRoutes = infos.filter(x => !x.analysis || x.analysis.length == 0);
        if (validRoutes.length == 0) {
            let path = this.options.controllerPaths.join(", ");
            this.log.newLine().error(`No valid controller found in [${path}]`);
            return false;
        }
        this.log.newLine().info("Routes generated successfully");
        this.log.info("--------------------------------------");
        validRoutes.forEach(x => {
            let method = "";
            switch (x.httpMethod) {
                case "GET":
                    method = "GET    ";
                    break;
                case "PUT":
                    method = "PUT    ";
                    break;
                case "PATCH":
                    method = "PATCH  ";
                    break;
                case "POST":
                    method = "POST   ";
                    break;
                case "DELETE":
                    method = "DELETE ";
                    break;
                case "EVENT":
                    method = "EVENT  ";
                    break;
            }
            this.log.info(`${method} ${x.route}`);
        });
        this.log.info("--------------------------------------");
        this.options.routeInfos = validRoutes;
        return true;
    }
    resolveValidators(option) {
        option.validators = validator_1.ValidatorFactory.resolve(option.validators, option.dependencyResolver);
    }
    /**
     * Initialize KambojaJS application
     * @returns HttpServer
     */
    init() {
        this.log = new logger_1.Logger(this.options.showLog);
        if (!this.isFolderProvided())
            throw new Error("Fatal error");
        this.storage.load(this.options.controllerPaths, "Controller");
        this.storage.load(this.options.modelPath, "Model");
        //TODO: possibly temporal coupling
        this.resolveValidators(this.options);
        let routeInfos = this.generateRoutes(this.storage.getFiles("Controller"));
        if (routeInfos.length == 0)
            throw new Error("Fatal error");
        if (!this.analyzeRoutes(routeInfos))
            throw new Error("Fatal Error");
        let httpControllers = routeInfos.filter(x => x.httpMethod != "EVENT");
        let httpApp = this.engine.init(httpControllers, this.options);
        this.set("httpApp", httpApp);
        let server = Http.createServer(httpApp);
        if (this.options.socketEngine) {
            let socketControllers = routeInfos.filter(x => x.httpMethod == "EVENT");
            let socketApp = this.options.socketEngine.init(socketControllers, this.options);
            this.set("socketApp", socketApp);
            socketApp.listen(server);
        }
        return server;
    }
}
Kamboja.defaultModelPath = "model";
exports.Kamboja = Kamboja;
